@@ -38,9 +38,7 @@ def main():
         'src/toil/provisioners/__init__.py',
         'src/toil/provisioners/node.py',
         'src/toil/provisioners/aws/boto2Context.py',
-        'src/toil/provisioners/aws/awsProvisioner.py',
         'src/toil/provisioners/aws/__init__.py',
-        'src/toil/batchSystems/slurm.py',
         'src/toil/batchSystems/gridengine.py',
         'src/toil/batchSystems/singleMachine.py',
         'src/toil/batchSystems/torque.py',
@@ -49,6 +47,7 @@ def main():
         'src/toil/batchSystems/lsf.py',
         'src/toil/batchSystems/__init__.py',
         'src/toil/batchSystems/abstractGridEngineBatchSystem.py',
+        'src/toil/batchSystems/awsBatch.py',
         'src/toil/batchSystems/lsfHelper.py',
         'src/toil/batchSystems/htcondor.py',
         'src/toil/batchSystems/mesos/batchSystem.py',
