@@ -24,15 +24,17 @@ sdistName = os.environ['_TOIL_SDIST_NAME']
 python = f'python{sys.version_info[0]}.{sys.version_info[1]}'
 pip = f'{python} -m pip'
 
+# Debian and Ubuntu don't package ensurepip by default so the python-venv package must be installed
+python_packages = {'python3.9': ['python3.9-distutils', 'python3.9-venv'],
+                   'python3.10': ['python3.10-distutils', 'python3.10-venv'],
+                   'python3.11': ['python3.11-distutils', 'python3.11-venv'],
+                   'python3.12': ['python3.12-venv'],
+                   'python3.13': ['python3.13-venv']}  # python3.13 removed distutils
 
-dependencies = ' '.join(['libffi-dev',  # For client side encryption for extras with PyNACL
+dependencies = ' '.join(python_packages[python] +
+                        ['libffi-dev',  # For client side encryption for extras with PyNACL
                          python,
                          f'{python}-dev',
-                         'python3.8-distutils' if python == 'python3.8' else '',
-                         'python3.9-distutils' if python == 'python3.9' else '',
-                         'python3.10-distutils' if python == 'python3.10' else '',
-                         'python3.11-distutils' if python == 'python3.11' else '',
-                         'python3.12-distutils' if python== 'python3.12' else '',
                          'python3-pip',
                          'libssl-dev',
                          'wget',
@@ -62,6 +64,11 @@ dependencies = ' '.join(['libffi-dev',  # For client side encryption for extras
                          'libapr1',
                          # Dependencies for singularity
                          'containernetworking-plugins',
+                         'libfuse2',
+                         'fuse',
+                         'fuse2fs',
+                         'uidmap',
+                         'squashfs-tools-ng',
                          # Dependencies for singularity on kubernetes
                          'tzdata'])
 
@@ -93,9 +100,6 @@ print(heredoc('''
 
     RUN if [ -z "$TARGETARCH" ] ; then echo "Specify a TARGETARCH argument to build this container"; exit 1; fi
 
-    # make sure we don't use too new a version of setuptools (which can get out of sync with poetry and break things)
-    ENV SETUPTOOLS_USE_DISTUTILS=stdlib
-
     # Try to avoid "Failed to fetch ...  Undetermined Error" from apt
     # See <https://stackoverflow.com/a/66523384>
     RUN printf 'Acquire::http::Pipeline-Depth "0";\\nAcquire::http::No-Cache=True;\\nAcquire::BrokenProxy=true;\\n' >/etc/apt/apt.conf.d/99fixbadproxy
@@ -107,11 +111,12 @@ print(heredoc('''
     # Find a repo with a Mesos build.
     # This one was archived like:
     # mkdir mesos-repo && cd mesos-repo
-    # wget --recursive --restrict-file-names=windows -k --convert-links --no-parent --page-requisites https://rpm.aventer.biz/Ubuntu/ https://www.aventer.biz/assets/support_aventer.asc https://rpm.aventer.biz/README.txt
+    # wget --recursive --restrict-file-names=windows -k --convert-links --no-parent --page-requisites -m https://rpm.aventer.biz/Ubuntu/ https://www.aventer.biz/assets/support_aventer.asc https://rpm.aventer.biz/README.txt
     # ipfs add -r .
-    RUN echo "deb https://public.gi.ucsc.edu/~anovak/outbox/toil/ipfs/QmeaErHzK4Dajz2mCMd36eUDQp7GX2bSECVRpGfrqdragR/rpm.aventer.biz/Ubuntu/focal focal main" \
+    # It contains a GPG key that will expire 2026-09-28
+    RUN echo "deb https://public.gi.ucsc.edu/~anovak/outbox/toil/ipfs/QmRXnGNiWk523zgNkuamENVkghMJ2zJtinVfgjHbc4Dcpr/rpm.aventer.biz/Ubuntu/focal focal main" \
         > /etc/apt/sources.list.d/mesos.list \
-        && curl https://public.gi.ucsc.edu/~anovak/outbox/toil/ipfs/QmeaErHzK4Dajz2mCMd36eUDQp7GX2bSECVRpGfrqdragR/www.aventer.biz/assets/support_aventer.asc | apt-key add -
+        && curl https://public.gi.ucsc.edu/~anovak/outbox/toil/ipfs/QmRXnGNiWk523zgNkuamENVkghMJ2zJtinVfgjHbc4Dcpr/www.aventer.biz/assets/support_aventer.asc | apt-key add -
 
     RUN apt-get -y update --fix-missing && \
         DEBIAN_FRONTEND=noninteractive apt-get -y upgrade && \
@@ -119,7 +124,7 @@ print(heredoc('''
         if [ $TARGETARCH = amd64 ] ; then DEBIAN_FRONTEND=noninteractive apt-get -y install mesos ; mesos-agent --help >/dev/null ; fi && \
         apt-get clean && \
         rm -rf /var/lib/apt/lists/*
-    
+
     # Install a particular old Debian Sid Singularity from somewhere.
     # It's 3.10, which is new enough to use cgroups2, but it needs a newer libc
     # than Ubuntu 20.04 ships. So we need a 22.04+ base.
@@ -141,6 +146,15 @@ print(heredoc('''
     # <https://github.com/apptainer/singularity/issues/6113#issuecomment-901897566>).
     #
     # So we need to make sure to install a downgraded squashfs first.
+    #
+    # TODO: Singularity has since resolved this on their end
+    # https://github.com/sylabs/singularity/pull/267
+    # This works by checking that the UID of the caller is not root
+    # In a Kubernetes pod, the default setup will have UID 0 even if the pod is unprivileged
+    # https://github.com/sylabs/singularity/issues/2727
+    # It is possible to avoid this by changing the user of the pod (such as runAsUser: 1000)
+    # but this may cause permission issues, ex: changed read/write permissions
+    # so the downgraded squashfs stays, but options for updated squashfs are possible
     ADD extra-debs.tsv /etc/singularity/extra-debs.tsv
     RUN wget -q "$(cat /etc/singularity/extra-debs.tsv | grep "^squashfs-tools.$TARGETARCH" | cut -f4)" && \
         dpkg -i squashfs-tools_*.deb && \
@@ -168,16 +182,18 @@ print(heredoc('''
     # The stock pip is too old and can't install from sdist with extras
     RUN curl -sS https://bootstrap.pypa.io/get-pip.py | {python}
 
-    # Default setuptools is too old
-    RUN {pip} install --upgrade setuptools==59.7.0
-
     # Include virtualenv, as it is still the recommended way to deploy pipelines
-    RUN {pip} install --upgrade virtualenv==20.0.17
+    RUN {pip} install --upgrade virtualenv==20.25.1
 
     # Install s3am (--never-download prevents silent upgrades to pip, wheel and setuptools)
+    # Install setuptools within the virtual environment to properly access distutils due to PEP 632 and gh-95299 in Python 3.12 release notes
+    # https://docs.python.org/3/whatsnew/3.12.html#summary-release-highlights
     RUN virtualenv --python {python} --never-download /home/s3am \
+        && /home/s3am/bin/pip install setuptools \
         && /home/s3am/bin/pip install s3am==2.0 \
         && ln -s /home/s3am/bin/s3am /usr/local/bin/
+    
+    RUN {pip} install --upgrade setuptools==69.2.0
 
     # Fix for https://issues.apache.org/jira/browse/MESOS-3793
     ENV MESOS_LAUNCHER=posix
@@ -215,4 +231,7 @@ print(heredoc('''
 
     RUN echo '[ ! -z "$TERM" -a -r /etc/motd ] && cat /etc/motd' >> /etc/bash.bashrc \
         && printf '{motd}' > /etc/motd
+    
+    # Final check that Toil actually runs at all in the container
+    RUN toil --version
 '''))
