@@ -7,11 +7,9 @@ import socket
 import subprocess
 import sys
 import time
-import typing
+from collections.abc import Iterator
 from contextlib import closing
-from typing import Iterator, List, Optional
-
-import pytz
+from typing import Optional
 
 logger = logging.getLogger(__name__)
 
@@ -23,19 +21,20 @@ def get_public_ip() -> str:
     try:
         # Try to get the internet-facing IP by attempting a connection
         # to a non-existent server and reading what IP was used.
-        ip = '127.0.0.1'
+        ip = "127.0.0.1"
         with closing(socket.socket(socket.AF_INET, socket.SOCK_DGRAM)) as sock:
             # 203.0.113.0/24 is reserved as TEST-NET-3 by RFC 5737, so
             # there is guaranteed to be no one listening on the other
             # end (and we won't accidentally DOS anyone).
-            sock.connect(('203.0.113.1', 1))
+            sock.connect(("203.0.113.1", 1))
             ip = sock.getsockname()[0]
         return ip
     except:
         # Something went terribly wrong. Just give loopback rather
         # than killing everything, because this is often called just
         # to provide a default argument
-        return '127.0.0.1'
+        return "127.0.0.1"
+
 
 def get_user_name() -> str:
     """
@@ -48,20 +47,23 @@ def get_user_name() -> str:
         except KeyError:
             # This is expected if the user isn't in /etc/passwd, such as in a
             # Docker container when running as a weird UID. Make something up.
-            return 'UnknownUser' + str(os.getuid())
+            return "UnknownUser" + str(os.getuid())
     except Exception as e:
         # We can't get the UID, or something weird has gone wrong.
-        logger.error('Unexpected error getting user name: %s', e)
-        return 'UnknownUser'
+        logger.error("Unexpected error getting user name: %s", e)
+        return "UnknownUser"
+
 
 def utc_now() -> datetime.datetime:
     """Return a datetime in the UTC timezone corresponding to right now."""
-    return datetime.datetime.utcnow().replace(tzinfo=pytz.UTC)
+    return datetime.datetime.utcnow().replace(tzinfo=datetime.timezone.utc)
+
 
 def unix_now_ms() -> float:
     """Return the current time in milliseconds since the Unix epoch."""
     return time.time() * 1000
 
+
 def slow_down(seconds: float) -> float:
     """
     Toil jobs that have completed are not allowed to have taken 0 seconds, but
@@ -79,9 +81,25 @@ def slow_down(seconds: float) -> float:
 
     return max(seconds, sys.float_info.epsilon)
 
-def printq(msg: str, quiet: bool) -> None:
+
+def printq(msg: str, quiet: bool, log: bool = False) -> None:
+    """
+    This is for functions used simultaneously in Toil proper and in the admin scripts.
+
+    Our admin scripts "print" to stdout, while Toil proper uses logging.  For a script that,
+    for example, cleans up IAM, EC2, etc. cruft leftover after failed CI runs, we can call
+    an AWS delete IAM role function, and this prints or logs progress (unless quiet is True),
+    depending on whether the function is called in, say, the jobstore or a script.
+
+    :param msg: The string to print or log to stdout.
+    :param quiet: Silent output to stdout.
+    :param log: Use logging (else "print" to the screen).
+    """
     if not quiet:
-        print(msg)
+        if not log:
+            print(msg)
+        else:
+            logger.debug(msg)
 
 
 def truncExpBackoff() -> Iterator[float]:
@@ -104,12 +122,23 @@ class CalledProcessErrorStderr(subprocess.CalledProcessError):
         if (self.returncode < 0) or (self.stderr is None):
             return str(super())
         else:
-            err = self.stderr if isinstance(self.stderr, str) else self.stderr.decode("ascii", errors="replace")
+            err = (
+                self.stderr
+                if isinstance(self.stderr, str)
+                else self.stderr.decode("ascii", errors="replace")
+            )
             return "Command '%s' exit status %d: %s" % (self.cmd, self.returncode, err)
 
 
-def call_command(cmd: List[str], *args: str, input: Optional[str] = None, timeout: Optional[float] = None,
-                useCLocale: bool = True, env: Optional[typing.Dict[str, str]] = None, quiet: Optional[bool] = False) -> str:
+def call_command(
+    cmd: list[str],
+    *args: str,
+    input: Optional[str] = None,
+    timeout: Optional[float] = None,
+    useCLocale: bool = True,
+    env: Optional[dict[str, str]] = None,
+    quiet: Optional[bool] = False
+) -> str:
     """
     Simplified calling of external commands.
 
@@ -140,14 +169,30 @@ def call_command(cmd: List[str], *args: str, input: Optional[str] = None, timeou
 
     logger.debug("run command: {}".format(" ".join(cmd)))
     start_time = datetime.datetime.now()
-    proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
-                            encoding='utf-8', errors="replace", env=env)
+    proc = subprocess.Popen(
+        cmd,
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+        encoding="utf-8",
+        errors="replace",
+        env=env,
+    )
     stdout, stderr = proc.communicate(input=input, timeout=timeout)
     end_time = datetime.datetime.now()
     runtime = (end_time - start_time).total_seconds()
     sys.stderr.write(stderr)
     if proc.returncode != 0:
-        logger.debug("command failed in {}s: {}: {}".format(runtime, " ".join(cmd), stderr.rstrip()))
-        raise CalledProcessErrorStderr(proc.returncode, cmd, output=stdout, stderr=stderr)
-    logger.debug("command succeeded in {}s: {}{}".format(runtime, " ".join(cmd), (': ' + stdout.rstrip()) if not quiet else ''))
+        logger.debug(
+            "command failed in {}s: {}: {}".format(
+                runtime, " ".join(cmd), stderr.rstrip()
+            )
+        )
+        raise CalledProcessErrorStderr(
+            proc.returncode, cmd, output=stdout, stderr=stderr
+        )
+    logger.debug(
+        "command succeeded in {}s: {}{}".format(
+            runtime, " ".join(cmd), (": " + stdout.rstrip()) if not quiet else ""
+        )
+    )
     return stdout
