@@ -15,9 +15,10 @@
 
 import logging
 import time
+from collections.abc import Iterable
 from queue import Empty, Queue
 from threading import Event, Thread
-from typing import Iterable, Optional, Set
+from typing import Optional
 
 from toil.job import ServiceJobDescription
 from toil.jobStores.abstractJobStore import AbstractJobStore
@@ -40,7 +41,7 @@ class ServiceManager:
 
         # These are all the client jobs that are waiting for their services to
         # start.
-        self.__waiting_clients: Set[str] = set()
+        self.__waiting_clients: set[str] = set()
 
         # This is used to terminate the thread associated with the service
         # manager
@@ -123,7 +124,9 @@ class ServiceManager:
             client_id = self.__clients_out.get(timeout=maxWait)
             self.__waiting_clients.remove(client_id)
             if self.__service_manager_jobs < 0:
-                raise RuntimeError("The number of jobs scheduled by the service manager cannot be negative.")
+                raise RuntimeError(
+                    "The number of jobs scheduled by the service manager cannot be negative."
+                )
             self.__service_manager_jobs -= 1
             return client_id
         except Empty:
@@ -141,7 +144,9 @@ class ServiceManager:
             client_id = self.__failed_clients_out.get(timeout=maxWait)
             self.__waiting_clients.remove(client_id)
             if self.__service_manager_jobs < 0:
-                raise RuntimeError("The number of jobs scheduled by the service manager cannot be negative.")
+                raise RuntimeError(
+                    "The number of jobs scheduled by the service manager cannot be negative."
+                )
             self.__service_manager_jobs -= 1
             return client_id
         except Empty:
@@ -157,7 +162,9 @@ class ServiceManager:
         try:
             service_id = self.__services_out.get(timeout=maxWait)
             if self.__service_manager_jobs < 0:
-                raise RuntimeError("The number of jobs scheduled by the service manager cannot be negative.")
+                raise RuntimeError(
+                    "The number of jobs scheduled by the service manager cannot be negative."
+                )
             self.__service_manager_jobs -= 1
             return service_id
         except Empty:
@@ -226,7 +233,7 @@ class ServiceManager:
 
         Will block until all services are started and blocked.
         """
-        logger.debug('Waiting for service manager thread to finish ...')
+        logger.debug("Waiting for service manager thread to finish ...")
         start_time = time.time()
         self.__terminate.set()
         self.__service_starter.join()
@@ -251,13 +258,17 @@ class ServiceManager:
         while True:
             with throttle(1.0):
                 if self.__terminate.is_set():
-                    logger.debug('Received signal to quit starting services.')
+                    logger.debug("Received signal to quit starting services.")
                     break
                 try:
                     client_id = self.__clients_in.get_nowait()
                     client = self.__toil_state.get_job(client_id)
                     host_id_batches = list(client.serviceHostIDsInBatches())
-                    logger.debug("Service manager processing client %s with %d batches of services", client, len(host_id_batches))
+                    logger.debug(
+                        "Service manager processing client %s with %d batches of services",
+                        client,
+                        len(host_id_batches),
+                    )
                     if len(host_id_batches) > 1:
                         # Have to fall back to the old blocking behavior to
                         # ensure entire service "groups" are issued as a whole.
@@ -288,7 +299,7 @@ class ServiceManager:
 
                 pending_service_count = len(starting_services)
                 if pending_service_count > 0 and log_limiter.throttle(False):
-                    logger.debug('%d services are starting...', pending_service_count)
+                    logger.debug("%d services are starting...", pending_service_count)
 
                 for service_id in list(starting_services):
                     service_job_desc = self._get_service_job(service_id)
@@ -297,7 +308,9 @@ class ServiceManager:
                         or service_job_desc.errorJobStoreID is None
                     ):
                         raise Exception("Must be a registered ServiceJobDescription")
-                    if not self.__job_store.file_exists(service_job_desc.startJobStoreID):
+                    if not self.__job_store.file_exists(
+                        service_job_desc.startJobStoreID
+                    ):
                         # Service has started (or failed)
                         logger.debug(
                             "Service %s has removed %s and is therefore started",
@@ -308,9 +321,13 @@ class ServiceManager:
                         client_id = service_to_client[service_id]
                         remaining_services_by_client[client_id] -= 1
                         if remaining_services_by_client[client_id] < 0:
-                            raise RuntimeError("The number of remaining services cannot be negative.")
+                            raise RuntimeError(
+                                "The number of remaining services cannot be negative."
+                            )
                         del service_to_client[service_id]
-                        if not self.__job_store.file_exists(service_job_desc.errorJobStoreID):
+                        if not self.__job_store.file_exists(
+                            service_job_desc.errorJobStoreID
+                        ):
                             logger.error(
                                 "Service %s has immediately failed before it could be used",
                                 service_job_desc,
@@ -321,13 +338,22 @@ class ServiceManager:
 
                 # Find if any clients have had *all* their services started.
                 ready_clients = set()
-                for client_id, remainingServices in remaining_services_by_client.items():
+                for (
+                    client_id,
+                    remainingServices,
+                ) in remaining_services_by_client.items():
                     if remainingServices == 0:
                         if client_id in clients_with_failed_services:
-                            logger.error('Job %s has had all its services try to start, but at least one failed', self.__toil_state.get_job(client_id))
+                            logger.error(
+                                "Job %s has had all its services try to start, but at least one failed",
+                                self.__toil_state.get_job(client_id),
+                            )
                             self.__failed_clients_out.put(client_id)
                         else:
-                            logger.debug('Job %s has all its services started', self.__toil_state.get_job(client_id))
+                            logger.debug(
+                                "Job %s has all its services started",
+                                self.__toil_state.get_job(client_id),
+                            )
                             self.__clients_out.put(client_id)
                         ready_clients.add(client_id)
                 for client_id in ready_clients:
@@ -344,7 +370,9 @@ class ServiceManager:
 
         # Start the service jobs in batches, waiting for each batch
         # to become established before starting the next batch
-        for service_job_list in self.__toil_state.get_job(client_id).serviceHostIDsInBatches():
+        for service_job_list in self.__toil_state.get_job(
+            client_id
+        ).serviceHostIDsInBatches():
             # When we get the job descriptions we store them here to go over them again.
             wait_on = []
             for service_id in service_job_list:
@@ -361,9 +389,13 @@ class ServiceManager:
                     service_job_desc.startJobStoreID,
                 )
                 if not self.__job_store.file_exists(service_job_desc.startJobStoreID):
-                    raise RuntimeError(f"Service manager attempted to start service {service_job_desc} that has already started")
+                    raise RuntimeError(
+                        f"Service manager attempted to start service {service_job_desc} that has already started"
+                    )
                 if not self.__toil_state.job_exists(str(service_job_desc.jobStoreID)):
-                    raise RuntimeError(f"Service manager attempted to start service {service_job_desc} that is not in the job store")
+                    raise RuntimeError(
+                        f"Service manager attempted to start service {service_job_desc} that is not in the job store"
+                    )
                 # At this point the terminateJobStoreID and errorJobStoreID
                 # could have been deleted, since the service can be killed at
                 # any time! So we can't assert their presence here.
@@ -382,7 +414,7 @@ class ServiceManager:
                     time.sleep(1.0)
 
                     if log_limiter.throttle(False):
-                        logger.info('Service %s is starting...', service_job_desc)
+                        logger.info("Service %s is starting...", service_job_desc)
 
                     # Check if the thread should quit
                     if self.__terminate.is_set():
@@ -395,9 +427,14 @@ class ServiceManager:
                     ):
                         # The service job has gone away but the service never flipped its start flag.
                         # That's not what the worker is supposed to do when running a service at all.
-                        logger.error('Service %s has completed and been removed without ever starting', service_job_desc)
+                        logger.error(
+                            "Service %s has completed and been removed without ever starting",
+                            service_job_desc,
+                        )
                         # Stop everything.
-                        raise RuntimeError(f"Service {service_job_desc} is in an inconsistent state")
+                        raise RuntimeError(
+                            f"Service {service_job_desc} is in an inconsistent state"
+                        )
 
                 # We don't bail out early here.
 
@@ -409,6 +446,5 @@ class ServiceManager:
                 # though, so they should stop immediately when we run them. TODO:
                 # this is a bad design!
 
-
         # Add the JobDescription to the output queue of jobs whose services have been started
         self.__clients_out.put(client_id)
