@@ -2,16 +2,19 @@
 import argparse
 import subprocess
 import sys
-from typing import List
 
 from configargparse import ArgumentParser
 
 
 def make_parser():
     p = ArgumentParser()
-    p.add_argument("progargs", nargs=argparse.REMAINDER, help="The program and its arguments")
+    p.add_argument(
+        "progargs", nargs=argparse.REMAINDER, help="The program and its arguments"
+    )
     p.add_argument("--num", type=int, help="number of times to run the application")
-    p.add_argument("--no-fail", help="add this flag to actually work", action="store_true")
+    p.add_argument(
+        "--no-fail", help="add this flag to actually work", action="store_true"
+    )
     return p
 
 
@@ -22,10 +25,12 @@ class Runner:
         else:
             self.indata = sys.stdin.read().encode(sys.stdin.encoding)
 
-    def run_once(self, args: List[str]):
-        subprocess.run(args, input=self.indata, stdout=sys.stdout, stderr=sys.stderr).check_returncode()
+    def run_once(self, args: list[str]):
+        subprocess.run(
+            args, input=self.indata, stdout=sys.stdout, stderr=sys.stderr
+        ).check_returncode()
 
-    def run_many(self, n: int, args: List[str]):
+    def run_many(self, n: int, args: list[str]):
         for i in range(n):
             self.run_once(args)
 
