@@ -14,7 +14,7 @@
 import logging
 import os
 import uuid
-from typing import Optional
+from typing import TYPE_CHECKING, Optional
 
 from toil.jobStores.aws.jobStore import AWSJobStore
 from toil.lib.aws.session import establish_boto3_session
@@ -29,11 +29,12 @@ logging.basicConfig(level=logging.DEBUG)
 class S3Test(ToilTest):
     """Confirm the workarounds for us-east-1."""
 
-    from mypy_boto3_s3 import S3ServiceResource
-    from mypy_boto3_s3.service_resource import Bucket
+    if TYPE_CHECKING:
+        from mypy_boto3_s3 import S3ServiceResource
+        from mypy_boto3_s3.service_resource import Bucket
 
-    s3_resource: Optional[S3ServiceResource]
-    bucket: Optional[Bucket]
+    s3_resource: Optional["S3ServiceResource"]
+    bucket: Optional["Bucket"]
 
     @classmethod
     def setUpClass(cls) -> None:
@@ -57,16 +58,22 @@ class S3Test(ToilTest):
         self.assertEqual(get_bucket_region(bucket_name), "us-east-1")
 
         # Make sure all the bucket location getting strategies work on a bucket we created
-        self.assertEqual(get_bucket_region(bucket_name, only_strategies = {1}), "us-east-1")
-        self.assertEqual(get_bucket_region(bucket_name, only_strategies = {2}), "us-east-1")
-        self.assertEqual(get_bucket_region(bucket_name, only_strategies = {3}), "us-east-1")
+        self.assertEqual(
+            get_bucket_region(bucket_name, only_strategies={1}), "us-east-1"
+        )
+        self.assertEqual(
+            get_bucket_region(bucket_name, only_strategies={2}), "us-east-1"
+        )
+        self.assertEqual(
+            get_bucket_region(bucket_name, only_strategies={3}), "us-east-1"
+        )
 
     def test_get_bucket_location_public_bucket(self) -> None:
         """
         Test getting buket location for a bucket we don't own.
         """
 
-        bucket_name = 'spacenet-dataset'
+        bucket_name = "spacenet-dataset"
         # This bucket happens to live in us-east-1
         self.assertEqual(get_bucket_region(bucket_name), "us-east-1")
 
