@@ -21,23 +21,25 @@ from toil.test import ToilTest
 logger = logging.getLogger(__name__)
 logging.basicConfig(level=logging.DEBUG)
 
+
 class TagGenerationTest(ToilTest):
     """
     Test for tag generation from environment variables
     """
+
     def test_build_tag(self):
         environment = dict()
         environment["TOIL_OWNER_TAG"] = "😀"
         environment["TOIL_AWS_TAGS"] = None
         tag_dict = build_tag_dict_from_env(environment)
-        assert(tag_dict == {'Owner': '😀'})
+        assert tag_dict == {"Owner": "😀"}
 
     def test_empty_aws_tags(self):
         environment = dict()
         environment["TOIL_OWNER_TAG"] = None
         environment["TOIL_AWS_TAGS"] = "{}"
         tag_dict = build_tag_dict_from_env(environment)
-        assert (tag_dict == dict())
+        assert tag_dict == dict()
 
     def test_incorrect_json_object(self):
         with pytest.raises(SystemExit):
@@ -58,7 +60,4 @@ class TagGenerationTest(ToilTest):
         environment["TOIL_OWNER_TAG"] = "😀"
         environment["TOIL_AWS_TAGS"] = '{"1": "2", " ":")"}'
         tag_dict = build_tag_dict_from_env(environment)
-        assert(tag_dict == {'Owner': '😀', '1': '2', ' ': ')'})
-
-
-
+        assert tag_dict == {"Owner": "😀", "1": "2", " ": ")"}
