@@ -25,21 +25,33 @@ class ProvisionerTest(ToilTest):
 
     def test_node_type_parsing(self) -> None:
         assert parse_node_types(None) == []
-        assert parse_node_types('') == []
-        assert parse_node_types('red beans') == [({'red beans'}, None)]
-        assert parse_node_types('red beans,rice') == [({'red beans'}, None), ({'rice'}, None)]
-        assert parse_node_types('red beans/black beans,rice') == [({'red beans', 'black beans'}, None), ({'rice'}, None)]
-        assert parse_node_types('frankfurters:0.05') == [({'frankfurters'}, 0.05)]
-        assert parse_node_types('red beans/black beans:999,rice,red beans/black beans') == [({'red beans', 'black beans'}, 999), ({'rice'}, None), ({'red beans', 'black beans'}, None)]
+        assert parse_node_types("") == []
+        assert parse_node_types("red beans") == [({"red beans"}, None)]
+        assert parse_node_types("red beans,rice") == [
+            ({"red beans"}, None),
+            ({"rice"}, None),
+        ]
+        assert parse_node_types("red beans/black beans,rice") == [
+            ({"red beans", "black beans"}, None),
+            ({"rice"}, None),
+        ]
+        assert parse_node_types("frankfurters:0.05") == [({"frankfurters"}, 0.05)]
+        assert parse_node_types(
+            "red beans/black beans:999,rice,red beans/black beans"
+        ) == [
+            ({"red beans", "black beans"}, 999),
+            ({"rice"}, None),
+            ({"red beans", "black beans"}, None),
+        ]
         with pytest.raises(ValueError):
-            parse_node_types('your thoughts:penny')
+            parse_node_types("your thoughts:penny")
         with pytest.raises(ValueError) as err:
-            parse_node_types(',,,')
-        assert 'empty' in str(err.value)
+            parse_node_types(",,,")
+        assert "empty" in str(err.value)
         with pytest.raises(ValueError):
-            parse_node_types('now hear this:')
+            parse_node_types("now hear this:")
         with pytest.raises(ValueError) as err:
-            parse_node_types('miles I will walk:500:500')
-        assert 'multiple' in str(err.value)
+            parse_node_types("miles I will walk:500:500")
+        assert "multiple" in str(err.value)
         with pytest.raises(ValueError):
-            parse_node_types('red beans:500/black beans:500,rice')
+            parse_node_types("red beans:500/black beans:500,rice")
