@@ -30,7 +30,7 @@ from toil.version import exactPython
 
 
 @contextmanager
-def tempFileContaining(content, suffix=''):
+def tempFileContaining(content, suffix=""):
     """
     Write a file with the given contents, and keep it on disk as long as the context is active.
     :param str content: The contents of the file.
@@ -38,7 +38,7 @@ def tempFileContaining(content, suffix=''):
     """
     fd, path = tempfile.mkstemp(suffix=suffix)
     try:
-        encoded = content.encode('utf-8')
+        encoded = content.encode("utf-8")
         assert os.write(fd, encoded) == len(encoded)
     except:
         os.close(fd)
@@ -52,41 +52,55 @@ def tempFileContaining(content, suffix=''):
 
 class ResourceTest(ToilTest):
     """Test module descriptors and resources derived from them."""
+
     def testStandAlone(self):
-        self._testExternal(moduleName='userScript', pyFiles=('userScript.py', 'helper.py'))
+        self._testExternal(
+            moduleName="userScript", pyFiles=("userScript.py", "helper.py")
+        )
 
     def testPackage(self):
-        self._testExternal(moduleName='foo.userScript', pyFiles=('foo/__init__.py',
-                                                                 'foo/userScript.py',
-                                                                 'foo/bar/__init__.py',
-                                                                 'foo/bar/helper.py'))
+        self._testExternal(
+            moduleName="foo.userScript",
+            pyFiles=(
+                "foo/__init__.py",
+                "foo/userScript.py",
+                "foo/bar/__init__.py",
+                "foo/bar/helper.py",
+            ),
+        )
 
     def testVirtualEnv(self):
-        self._testExternal(moduleName='foo.userScript',
-                           virtualenv=True,
-                           pyFiles=('foo/__init__.py',
-                                    'foo/userScript.py',
-                                    'foo/bar/__init__.py',
-                                    'foo/bar/helper.py',
-                                    'de/pen/dency.py',
-                                    'de/__init__.py',
-                                    'de/pen/__init__.py'))
+        self._testExternal(
+            moduleName="foo.userScript",
+            virtualenv=True,
+            pyFiles=(
+                "foo/__init__.py",
+                "foo/userScript.py",
+                "foo/bar/__init__.py",
+                "foo/bar/helper.py",
+                "de/pen/dency.py",
+                "de/__init__.py",
+                "de/pen/__init__.py",
+            ),
+        )
 
     def testStandAloneInPackage(self):
-        self.assertRaises(ResourceException,
-                          self._testExternal,
-                          moduleName='userScript',
-                          pyFiles=('__init__.py', 'userScript.py', 'helper.py'))
+        self.assertRaises(
+            ResourceException,
+            self._testExternal,
+            moduleName="userScript",
+            pyFiles=("__init__.py", "userScript.py", "helper.py"),
+        )
 
     def _testExternal(self, moduleName, pyFiles, virtualenv=False):
         dirPath = self._createTempDir()
         if virtualenv:
             self.assertTrue(inVirtualEnv())
             # --never-download prevents silent upgrades to pip, wheel and setuptools
-            subprocess.check_call(['virtualenv', '--never-download', '--python', exactPython, dirPath])
-            sitePackages = os.path.join(dirPath, 'lib',
-                                        exactPython,
-                                        'site-packages')
+            subprocess.check_call(
+                ["virtualenv", "--never-download", "--python", exactPython, dirPath]
+            )
+            sitePackages = os.path.join(dirPath, "lib", exactPython, "site-packages")
             # tuple assignment is necessary to make this line immediately precede the try:
             oldPrefix, sys.prefix, dirPath = sys.prefix, dirPath, sitePackages
         else:
@@ -95,21 +109,23 @@ class ResourceTest(ToilTest):
             for relPath in pyFiles:
                 path = os.path.join(dirPath, relPath)
                 os.makedirs(os.path.dirname(path), exist_ok=True)
-                with open(path, 'w') as f:
-                    f.write('pass\n')
+                with open(path, "w") as f:
+                    f.write("pass\n")
             sys.path.append(dirPath)
             try:
                 userScript = importlib.import_module(moduleName)
                 try:
-                    self._test(userScript.__name__,
-                               expectedContents=pyFiles,
-                               allowExtraContents=True)
+                    self._test(
+                        userScript.__name__,
+                        expectedContents=pyFiles,
+                        allowExtraContents=True,
+                    )
                 finally:
                     del userScript
                     while moduleName:
                         del sys.modules[moduleName]
                         self.assertFalse(moduleName in sys.modules)
-                        moduleName = '.'.join(moduleName.split('.')[:-1])
+                        moduleName = ".".join(moduleName.split(".")[:-1])
 
             finally:
                 sys.path.remove(dirPath)
@@ -120,17 +136,22 @@ class ResourceTest(ToilTest):
     def testBuiltIn(self):
         # Create a ModuleDescriptor for the module containing ModuleDescriptor, i.e. toil.resource
         module_name = ModuleDescriptor.__module__
-        self.assertEqual(module_name, 'toil.resource')
+        self.assertEqual(module_name, "toil.resource")
         self._test(module_name, shouldBelongToToil=True)
 
-    def _test(self, module_name,
-              shouldBelongToToil=False, expectedContents=None, allowExtraContents=True):
+    def _test(
+        self,
+        module_name,
+        shouldBelongToToil=False,
+        expectedContents=None,
+        allowExtraContents=True,
+    ):
         module = ModuleDescriptor.forModule(module_name)
         # Assert basic attributes and properties
         self.assertEqual(module.belongsToToil, shouldBelongToToil)
         self.assertEqual(module.name, module_name)
         if shouldBelongToToil:
-            self.assertTrue(module.dirPath.endswith('/src'))
+            self.assertTrue(module.dirPath.endswith("/src"))
 
         # Before the module is saved as a resource, localize() and globalize() are identity
         # methods. This should log.warnings.
@@ -139,27 +160,32 @@ class ResourceTest(ToilTest):
         # Create a mock job store ...
         jobStore = MagicMock()
         # ... to generate a fake URL for the resource ...
-        url = 'file://foo.zip'
+        url = "file://foo.zip"
         jobStore.getSharedPublicUrl.return_value = url
         # ... and save the resource to it.
         resource = module.saveAsResourceTo(jobStore)
         # Ensure that the URL generation method is actually called, ...
-        jobStore.getSharedPublicUrl.assert_called_once_with(sharedFileName=resource.pathHash)
+        jobStore.getSharedPublicUrl.assert_called_once_with(
+            sharedFileName=resource.pathHash
+        )
         # ... and that ensure that write_shared_file_stream is called.
-        jobStore.write_shared_file_stream.assert_called_once_with(shared_file_name=resource.pathHash,
-                                                                  encrypted=False)
+        jobStore.write_shared_file_stream.assert_called_once_with(
+            shared_file_name=resource.pathHash, encrypted=False
+        )
         # Now it gets a bit complicated: Ensure that the context manager returned by the
         # jobStore's write_shared_file_stream() method is entered and that the file handle yielded
         # by the context manager is written to once with the zipped source tree from which
         # 'toil.resource' was originally imported. Keep the zipped tree around such that we can
         # mock the download later.
-        file_handle = jobStore.write_shared_file_stream.return_value.__enter__.return_value
+        file_handle = (
+            jobStore.write_shared_file_stream.return_value.__enter__.return_value
+        )
         # The first 0 index selects the first call of write(), the second 0 selects positional
         # instead of keyword arguments, and the third 0 selects the first positional, i.e. the
         # contents. This is a bit brittle since it assumes that all the data is written in a
         # single call to write(). If more calls are made we can easily concatenate them.
         zipFile = file_handle.write.call_args_list[0][0][0]
-        self.assertTrue(zipFile.startswith(b'PK'))  # the magic header for ZIP files
+        self.assertTrue(zipFile.startswith(b"PK"))  # the magic header for ZIP files
 
         # Check contents if requested
         if expectedContents is not None:
@@ -186,7 +212,7 @@ class ResourceTest(ToilTest):
             # urlopen() that yields the zipped tree ...
             mock_urlopen = MagicMock()
             mock_urlopen.return_value.read.return_value = zipFile
-            with patch('toil.resource.urlopen', mock_urlopen):
+            with patch("toil.resource.urlopen", mock_urlopen):
                 # ... and use it to download and unpack the resource
                 localModule = module.localize()
             # The name should be equal between original and localized resource ...
@@ -217,22 +243,27 @@ class ResourceTest(ToilTest):
             def fn():
                 pass
 
-            if __name__ == '__main__':
+            if __name__ == "__main__":
                 parser = ArgumentParser()
                 Job.Runner.addToilOptions(parser)
                 options = parser.parse_args()
-                job = Job.wrapFn(fn, memory='10M', cores=0.1, disk='10M')
+                job = Job.wrapFn(fn, memory="10M", cores=0.1, disk="10M")
                 with Toil(options) as toil:
                     toil.start(job)
 
-        scriptBody = dedent('\n'.join(getsource(script).split('\n')[1:]))
-        shebang = '#! %s\n' % sys.executable
+        scriptBody = dedent("\n".join(getsource(script).split("\n")[1:]))
+        shebang = "#! %s\n" % sys.executable
         with tempFileContaining(shebang + scriptBody) as scriptPath:
-            self.assertFalse(scriptPath.endswith(('.py', '.pyc')))
+            self.assertFalse(scriptPath.endswith((".py", ".pyc")))
             os.chmod(scriptPath, 0o755)
-            jobStorePath = scriptPath + '.jobStore'
-            process = subprocess.Popen([scriptPath, jobStorePath], stderr=subprocess.PIPE)
+            jobStorePath = scriptPath + ".jobStore"
+            process = subprocess.Popen(
+                [scriptPath, jobStorePath], stderr=subprocess.PIPE
+            )
             stdout, stderr = process.communicate()
-            self.assertTrue('The name of a user script/module must end in .py or .pyc.' in stderr.decode('utf-8'))
+            self.assertTrue(
+                "The name of a user script/module must end in .py or .pyc."
+                in stderr.decode("utf-8")
+            )
             self.assertNotEqual(0, process.returncode)
             self.assertFalse(os.path.exists(jobStorePath))
