@@ -31,7 +31,9 @@ def internet_connection() -> bool:
 
 def main() -> None:
     if not internet_connection():
-        raise RuntimeError('No internet.  Updating the EC2 Instance list requires internet.')
+        raise RuntimeError(
+            "No internet.  Updating the EC2 Instance list requires internet."
+        )
     updateStaticEC2Instances()
 
 
